/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __CONFIG_H__
#define __CONFIG_H__

#include <qstring.h>
#include <qstrlist.h>
#include <qlist.h>
#include <qfile.h>
//#include "plugins/inModule.h"

struct fileinfo
{
	char title[40]; // description of the filetype
	char recognition[2048]; // recognitions patterns (must be separated by \n)
	char doubleclick[2048]; // Actions to execute (must be separated by \n)
	char doubleclickshift[2048]; // Actions to execute (must be separated by \n)
	char doubleclickctrl[2048]; // Actions to execute (must be separated by \n)
	char doubleclickalt[2048]; // Actions to execute (must be separated by \n)

	unsigned int mincheck; // for internal use only (don't touch it !!!!)
	unsigned int maxcheck; // for internal use only (don't touch it !!!!)
};

class inComponent;

class CConfig
{
	public:
		CConfig(QString file);
		// open the selected file

		void setModule(inComponent *mod);

		fileinfo *getByIndex(const unsigned int index) { return ((index >= nb) ? (0) : (fileInfo + index)); }
		// return the filetype informations corresponding to the index

		fileinfo *get(const char *file);
		// try to detect the filetype of 'file' argument
		// and returns corresponding informations
		// returns 0 if no filetype can be found

		int addEntry();
		// add a new filetype titles 'untitled'
		// and return the corresponding index

		void removeEntry(int ind);
		// remove the entry corresponding to the 'ind' index

		char *getNextTitle();
		// returns the next filetype
		// or 0 if no more informations are present
		// this function is useful when you want to have a list of all the filetypes

		void save();
		// save the configuration to disk

		~CConfig() { save(); }
		// save the configuration to disk

	private:

		inComponent *module;

		void readFile();
		// to read the content of the file
		
		enum check_t { MATCHBYTE = 0, MATCHCHAR, MATCHCHARNOCASE, MATCHNAME, MOVETO, MOVE, NOACTION };

		struct checkinfo
		{
			check_t action; // recognition type ( example : MATCH)
			char value[2048]; // value of the check ( example "32 44")
		};

		void parseRecognition(const char *, const char *, int *, int *);
		// will parse the string recognition and reformat it
		// this will prevent fonction like Matchx to do it in realtime 
		// and so it might speed up things a lot !!

		int checkMatchByte(const char *, QString);
		int checkMatchName(const char *, QString);
		int checkMatchCharacter(const char *, QString);
		int checkMatchCharacterNoCase(const char *, QString);
		int checkMoveTo(const char *, QString);

		QStrList total;
		
		unsigned int indiceRenvoie;
		// index returned by the getNextTitle function

		bool changed;
		// to know if we must rewrite the configuration file when quitting

		char buffer[2048];
		
		QString localName;

		bool alreadyOpened;
		
		unsigned int nb;
		
		unsigned int nbCheck;
		
		fileinfo *fileInfo, *parcours;

		checkinfo *checkInfo;

		bool cantOpen;
		// if a file can't be accessed, no need to reopen it many times !!
};

#endif
