/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qfile.h>
#include <qmessagebox.h>	
#include <stdlib.h> // realloc()
#include <ctype.h> // tolower()
#include <string.h> // strtok()
//#include <unistd.h> // unlink()
#include "configOpt.h"


CConfigOpt::CConfigOpt(QString name, QDir current)
{
  localName = name;
  localDir = current.absPath();
  readFile();
}


void CConfigOpt::readFile()
{
	QFile file(localName.data());
	file.open( IO_ReadWrite );
	// the file is created if it doesn't exist


	//currentMin = currentMax = -1;

	nb = nbCheck = 0;

	QString tmp;



	// now we add all the lines of the file in the chained list
	while(file.atEnd() == FALSE)
	{
		file.readLine(buffer, 2048);
		buffer[strlen(buffer) - 1] = 0;

		// if it's a commented line we throw it away !!!
		if (buffer[0] != '#')
		{
			//debug("Premier cars de la chaine : %d %d", buffer[0], buffer[1]);
			if (buffer[0] != 0)
			{
			  	total.append(buffer);

			}
		}
	}	
	file.close();

	if (total.count() == 0) // the file was empty, we fill it with default values
	{
	  //total.append("texteditor=vi");
	  total.append("hexeditor=ghex");
	  QString tmp = "pluginpath=";
	  tmp += localDir;
	  tmp += "/plugins/";
	  total.append(tmp.data());
	}  
	  
}

int CConfigOpt::addEntry(QString title, QString value)
{
  //debug("addEntry()");
  QString toInsert(title);
  toInsert += "=";
  toInsert += value;

  QString found;
  int i;

  int count = total.count();

  // first, we try to find if such an entry already exists in the list
  for(i = 0 ; i < count ; i++)
  { 
    found = total.at(i);
    if (found.find(title + "=") == 0) // an entry has been found, we remove it
    {
      total.remove(i);
      break;
    }
  }  
  //if (total.find
  total.insert(i, toInsert);

	return i;
}

bool CConfigOpt::removeEntry(QString title)
{
  QString toSearch(title);
  toSearch += "=";
  char *buffer = new char[2048];
  int count = total.count();
  for(int i = 0 ; i < count ; i++)
  {
    buffer = total.at(i);
    //debug("%s", buffer);
    QString tmp(buffer);
    if (tmp.find(toSearch) == 0) // we found what we want
    {
      total.remove(i); // so we remove it from the list
      return TRUE;  
    }  
  }
  return FALSE;
}	   

QString CConfigOpt::get(QString title, QString replacement)
{
  QString toSearch(title);
  toSearch += "=";
  char *buffer = new char[2048];
  int count = total.count();
  for(int i = 0 ; i < count ; i++)
  {
    buffer = total.at(i);
    //debug("%s", buffer);
    QString tmp(buffer);
    if (tmp.find(toSearch) == 0) // we found what we want
    {
      QString toReturn(tmp);
      toReturn = toReturn.right(toReturn.length() - toReturn.find('=') - 1);
      //debug("Valeur a renvoyer : %s", toReturn.data());
      return toReturn;
    }  
    //total.remove(i); // so we remove it from the list
  }  

  return replacement;
}

void CConfigOpt::save()
{
  	//debug("save du fichier");
	if (QFile::remove(localName.data()) == FALSE)
	{
		debug("Warning : could not delete the file %s, aborting", localName.data());
		return;
	}

	QFile file(localName.data());	
	file.open(IO_WriteOnly);

	int count = total.count();
//debug("to be written :");
	for(int i = 0 ; i < count ; i++)
	{
//debug("%s", total.at(i));
	  file.writeBlock(total.at(i), strlen(total.at(i)));
	  file.writeBlock("\n", 1);
	
	}

	file.close();	

	// let's reread the file
	readFile();
}
