/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __CONFIGFT_H__
#define __CONFIGFT_H__

#include <qstring.h>
#include <qstrlist.h>
#include <qlist.h>
#include <qfile.h>
#include <qdir.h>


class CConfigOpt
{
	public:
		CConfigOpt(QString file, QDir current);
		// Open the selected file
		// note : the current parameter is the zfm's current directory
		// useful for default plugins directory
		
		void clear() { total.clear(); }
		// clear all the options
		// you must fill it again after that !

		int addEntry(QString title, QString value);
		// add a new entry in the options
		// and return the index of this entry
		// if an entry with that tilte already exists, it's replaced

		bool removeEntry(QString title);
		// remove the entry corresponding to the title

		QString get(QString title, QString replacement);
		// get the entry corresponding to the title
		// if it doesn't exist, return the replacement value

		void save();
		// save all the entries on disk

		~CConfigOpt() {}
		// save all the entries on disk

	private:

		void readFile();
		// read the configuration file
		// for example at the beginning (in the constructor or after having saved the file)

		QStrList total;
		// list containing all the entries
		
		bool changed;
		// to know if we must rewrite the file when quitting

		char buffer[2048];
		// buffer for reading access

		QString localName;
		QString localDir;

		QFile fileOpened;

		unsigned int nb;
		
		unsigned int nbCheck;
		
};

#endif
