/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qapplication.h>
#include <stdlib.h> // realloc()
#include "myListBox.h"

myListBox::myListBox(QWidget *parent, const char *name)
	: QListBox(parent, name)
{
	Entry = 0;
	nb = 0;

	check.loadFromData(IDB_CHECK, sizeof(IDB_CHECK) / sizeof(unsigned char));
	check_mask.loadFromData(IDB_CHECK_MASK, sizeof(IDB_CHECK_MASK) / sizeof(unsigned char));
	
	check.setMask(check_mask);

	current = -1;
}


void myListBox::addEntry(QString entree)
{
	insertItem(entree.data());
	nb++;
	Entry = (entry *) realloc(Entry, nb * sizeof(entry));
	(Entry + (nb - 1)) -> selected = FALSE;
	strcpy((Entry + (nb - 1)) -> text, entree.data());
	//repaint();
}	


void myListBox::deSelect(int indice)
{
	if (indice >= 0) // un seul element a deselectionner
		(Entry + indice) -> selected = FALSE;
	else // tous les elements
	{
		for (int i = 0 ; i < nb ; i++)
			(Entry + i) -> selected = FALSE;
	}

	repaint();
}


void myListBox::paintCell(QPainter *p, int row, int col)
{
	int w = cellWidth(row);
	int h = cellHeight(row);

	if (row == current)
		p -> fillRect(0, 0, w, h, QApplication::winStyleHighlightColor());

	p -> drawText(16, 0, w, h, AlignVCenter, (Entry + row) -> text);

	if (isSelected(row))
		p -> drawPixmap(2, 1, check);

}


void myListBox::mousePressEvent(QMouseEvent *e)
{
	QPoint clickedPos = e -> pos();
	int clickedRow = findRow(clickedPos.y());

	if (clickedRow >= 0);
	{
		current = clickedRow;
		repaint();
		emit(clicked(clickedRow));
	}
}


void myListBox::mouseDoubleClickEvent(QMouseEvent *e)
{
	QPoint clickedPos = e -> pos();
	int clickedRow = findRow(clickedPos.y());

	if (clickedRow >= 0);
		emit(dblClicked(clickedRow));
}


void myListBox::setSelected(int indice, bool b)
{
	(Entry + indice) -> selected = b;
	updateCell(0, indice);
}
