/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qapplication.h>
#include <stdlib.h> // realloc()
#include "pixmapListBox.h"
#include "../toolbar.h" // to obtain all the pictures

pixmapListBox::pixmapListBox(QWidget *parent, const char *name)
	: QListBox(parent, name)
{
	nb = 0;

	for(int i = 0 ;; i++)
	{
	    if (pixmapInfo[i].size == 0) break;
	    nb++;
	    insertItem(pixmapInfo[i].name);
	}

	current = -1;
	setCellHeight(0);
}

int pixmapListBox::cellHeight(int num)
{
    return 20;
}


void pixmapListBox::paintCell(QPainter *p, int row, int col)
{
	int w = cellWidth(row);
	int h = cellHeight(row);

	QPixmap pi;
	pi.loadFromData(pixmapInfo[row].pixmap, pixmapInfo[row].size);

	if (row == current)
		p -> fillRect(0, 0, w, h, QApplication::winStyleHighlightColor());

	p -> drawText(28, 0, w, h, AlignVCenter, pixmapInfo[row].name);
	p -> drawPixmap(1, 2, pi);

}


void pixmapListBox::mousePressEvent(QMouseEvent *e)
{
	QPoint clickedPos = e -> pos();
	int clickedRow = findRow(clickedPos.y());

	if (clickedRow >= 0);
	{
		int newCurrent = clickedRow;
		if (newCurrent != current)
		{
		    int oldCurrent = current;
		    current = newCurrent;

		    updateItem(current);
		    updateItem(oldCurrent);
		}

		current = clickedRow;
		//repaint();
		emit(clicked(clickedRow));
	}
}

void pixmapListBox::mouseDoubleClickEvent(QMouseEvent *e)
{
	emit(dblClicked(current));
}
