/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __ZLIST_H__
#define __ZLIST_H__

#include <qlist.h>
#include "fileinfo.h"

typedef QList<ZFileInfo> ZListBase;
typedef QList<ZFileInfoDevice> ZListBaseDevice;

class ZList : public ZListBase
{
    public:
	ZList( bool deepCopies = FALSE ) { dc = deepCopies; }
	ZList( const ZList & );
	~ZList() { clear(); }
	ZList& operator=( const ZList & );

    private:
	GCI newItem( GCI d ) { return dc ? new ZFileInfo(*(ZFileInfo *)d) : d; }
	void deleteItem ( GCI d ) { if ( dc ) delete[] (ZFileInfo *)d; }
	
	bool dc;
};


inline ZList & ZList::operator =( const ZList &orig )
{
    clear();
    dc = orig.dc;
    ZListBase::operator=(orig);
    return *this;
}


inline ZList::ZList( const ZList &orig )
{
    dc = FALSE;
    operator=(orig);
}


class ZListDevice : public ZListBaseDevice
{
    public:
	ZListDevice( bool deepCopies = TRUE ) { dc = deepCopies; }
	ZListDevice( const ZListDevice & );
	~ZListDevice() { clear(); }
	ZListDevice& operator=( const ZListDevice & );

    private:
	GCI newItem( GCI d ) { return dc ? new ZFileInfoDevice(*(ZFileInfoDevice *)d) : d; }
	void deleteItem ( GCI d ) { if ( dc ) delete[] (ZFileInfoDevice *)d; }
	
	bool dc;
};


inline ZListDevice & ZListDevice::operator =( const ZListDevice &orig )
{
    clear();
    dc = orig.dc;
    ZListBaseDevice::operator=(orig);
    return *this;
}


inline ZListDevice::ZListDevice( const ZListDevice &orig )
{
    dc = FALSE;
    operator=(orig);
}

#endif
