// classe ZListBox
// (c)1998 GOUIRAN Christophe


#ifndef ZLISTBOX_H
#define ZLISTBOX_H

#include <qtablevw.h> // QTableView
#include <qdir.h> // QDir
#include <qscrbar.h> // QScrollBar
#include <qheader.h> // class QHeader
#include <qfont.h>
#include <qfontmetrics.h>
#include "outModule.h"
#include <qstrlist.h>
#include "../list.h" // class ZFileInfo

#define COLS 8 // nombre de colonnes

class widget;

class ZListBox : public outComponent
{
	Q_OBJECT
	public:
		ZListBox(CConfig *config, QWidget *parent = 0, const char *name = 0);
	
		void display(ZList &liste);
		void displayDevices(ZListDevice &liste);

		QStrList getMenu();
		// retourne la liste des entrees a placer dans le menu

	protected:
		void resizeEvent(QResizeEvent *);

		QHeader *header;
		widget *list;
	
		bool scanning;

		bool visible[COLS]; // pour savoir si un champ doit etre affiche
		int largeur[COLS]; // taille des cellules
		bool autoUpdate; // pour savoir si la taille des cellules se met a jour automatiquement apres un scan

    signals:
		void dblClicked(QString, int);
		// double click (accompagne de son qualifier)

   		void highlighted(QString);
		// le pointeur de la souris est sur un fichier

		void clicked(QString, int);
		// simple click

	public slots:
		void menuClicked(int);

		void sizeChange(int, int);
		// recu par le widget listbox quand il faut retailler un header (calcul automatique de la largeur
		// des headers)

		void colorChanged(int id, QColor color);
		// recu quand une couleur a ete changee par les preferences	

		void sliderReleased();
		// recu quand on relache un slider

};


class widget : public QTableView
{
    Q_OBJECT
    public:

		widget(CConfig *config, QWidget *parent, const char *name = 0);	
    
		void display(ZList &list, bool, bool);
		// permet d'afficher le contenu de la liste
		// le deuxieme parametre est mis pour savoir s'il doit y avoir le calcul automatique de la largeur
		// des cellules


		void displayDevices(ZListDevice &, bool);
		// affiche la liste des devices
      
		void changeColor(int id, QColor color);

		void sliderReleased();

		QColor getColor(int id) { return color[id]; }

    protected:

		int cellWidth(int n);
		// renvoie la taille maximale de la colonne n
		// (utile pour les colonnes qui changent de taille automatiquement
		// comme des grandes :) )

		int cellHeight(int n);
		// idem mais pour la hauteur

		int totalWidth();
		// renvoie la taille d'une ligne entiere
	
		int totalHeight();
		// renvoie la hauteur totale de toutes les colonnes cumulees

		void deSelect();
		//deselectionne tous les fichiers

		// slots normaux
		void paintCell( QPainter*, int row, int col );
   		void mousePressEvent( QMouseEvent* );
		void mouseMoveEvent(QMouseEvent*);
		void leaveEvent(QEvent *);
   		void mouseDoubleClickEvent(QMouseEvent *);
		void keyPressEvent( QKeyEvent* );
   		void focusInEvent( QFocusEvent* );
   		void focusOutEvent( QFocusEvent* );
		void resizeEvent(QResizeEvent *);
	
		const QScrollBar *sb;
	
    signals:
		void dblClicked(QString, int);
		// double click

   		void highlighted(QString);
		// le pointeur de la souris est sur un fichier

		void clicked(QString, int);
		// simple click

		void offsetChanged(int);
		// quand on touche aux en-tete

		void changeSizeHeader(int, int);
		// envoye quand on veut prevenir la listbox qu'il faut changer la taille d'un header 	

	public slots:
		void sizeChange(int, int, int);
		// recu quand on bouge l'en-tete

    private:
	
		void Calcul(const ZFileInfo *fi);
		// permet de determiner la plus grande hauteur qu'il faut appliquer aux cellules

		QDir dir;

		ZList contents;
		ZListDevice contentsDevices;
	
		int nombre; // nombre de fichiers enregistres dans les listes

		int numSelected;
		// nombre de fichiers selectionnes

		QFont font;
		QFontMetrics *fm;

		int curRow;
   		int curCol;
		int NombreCol; // nombre de colonnes a afficher
		int taille[COLS]; // largeur de chaque celule

		int lastClickedX, lastClickedY;
		// dernier endroit ou on  a clicke avant d'appuyer sur SHIFT
	
		bool toBeUpdated;
		// permet de savoir si apres un paintCell la hauteur des cellules doit changer
		// en effet, je ne peux pas le faire en temps reel desole :(

		CConfig *localConfig;

		QStrList fileTypes;

		int toBeResized;
		// valeur correspondante

		enum mode_t {FILES, DEVICES};
		// qu'est ce qu'on doit afficher dans la liste

		mode_t mode;

		QColor color[9];
		// ensemble des couleurs utilisees
		// respectivement :
		// couleur de fond
		// couleur de texte des fichiers
		// couleur de texte des fichiers selectionnes
		// couleur de fond des fichiers
		// couleur de fond des fichiers selectionnee
		// ensuite idem pour les repertoires
};

#endif
