/*
View Plugin for The ZFM
display icon view
Nicolas Roard, 1999
*/

#ifndef __DIRICN_H__
#define __DIRICN_H__

#include <qtablevw.h> // QTableView
#include <qdir.h> // QDir
#include <qscrbar.h> // QScrollBar
#include <qheader.h> // class QHeader
#include <qfont.h>
#include <qfontmetrics.h>
#include <qtableview.h>
#include <qpixmap.h>
#include <qstrlist.h>
#include <stdlib.h>
#include "../list.h" // class ZFileInfo
#include "../configOpt.h"
#include "outModule.h"

class DirIcn;

class MonPlug : public outComponent
{
    Q_OBJECT

public:
    MonPlug (CConfig *config, QWidget *parent = 0, const char *name = 0);
    void display(ZList &liste);
    void displayDevices(ZListDevice &liste);
    QStrList getMenu();

protected:
    void resizeEvent(QResizeEvent *);
    DirIcn *widget;

signals:
    void dblClicked(QString, int);
    // double click (accompagne de son qualifier)
    void highlighted(QString);
    // le pointeur de la souris est sur un fichier
    void clicked(QString, int);
    // simple click


public slots:
    void menuClicked(int);
};


class listView
{
public:
    listView(ZList &list);
    ~listView();
    int setList (ZList &list);
    int getType(QString Name);
    int getType(int Num);
    QString getName(int Num);
    QString getPath(int Num);
    int getNbFiles();
    int getOldRow(int Num);
    int getOldCol(int Num);
    int setOldRow(int Num, int row);
    int setOldCol(int Num, int col);

private:
    int m_NbFiles;
    int* m_listType;
    int* m_oldRow;
    int* m_oldCol;
    QString* m_fileNames;
    QString* m_Paths;
};


class DirIcn : public QTableView
{
    Q_OBJECT

public:
    DirIcn (CConfig *config, QWidget* parent = 0, const char* name = 0 );
    void display(ZList &liste);
    void displayDevices(ZListDevice &liste);
    QStrList getMenu();
    void setLargeur (int largeur);
    void setHauteur (int hauteur);
    int getLargeur ();
    int getHauteur ();
    void setWidthIcones (int width);
    void setHeightIcones (int height);
    void updateSize();

signals:
    void dblClicked(QString, int);
    // double click (accompagne de son qualifier)
    void highlighted(QString);
    // le pointeur de la souris est sur un fichier
    void clicked(QString, int);
    // simple click

public slots:
    void menuClicked(int);
    //void sizeChange(int, int);
    // recu par le widget listbox quand il faut retailler un header (calcul automatique de la largeur
    // des headers)
    void colorChanged(int id, QColor color);
    // recu quand une couleur a ete changee par les preferences
    void sliderReleased();
    // recu quand on relache un slider

private:
    void mouseMoveEvent( QMouseEvent* );
    void mouseDoubleClickEvent ( QMouseEvent* );
    void mousePressEvent(class QMouseEvent *);
    void keyPressEvent( QKeyEvent* );
    void focusInEvent( QFocusEvent* );
    void focusOutEvent( QFocusEvent* );
    void resizeEvent(QResizeEvent *r);
    void paintCell (QPainter*p, int, int);

    CConfigOpt* configOptions;

    listView* IconList;
    ZList contents;

    int curRow;
    int curCol;
    int m_Largeur;
    int m_Hauteur;
    bool FocusMouse;
    bool OneClick;

    QPixmap* PIXfolderClose ;
    QPixmap* PIXfolderOpen ;
    QPixmap* PIXfolderLink ;
    QPixmap* PIXfileText ;
    QPixmap* PIXfileExec ;
    QPixmap* PIXfileLink ;

    QPixmap* PIXfolderRoot ;
    QPixmap* PIXfolderParent ;
    QPixmap* PIXfolderHome ;
    QPixmap* PIXfileCore ;
    QPixmap* PIXfileDefault ;
};

#endif      // __DIRICN_H__
