// Classe ZListeur du projet zfm
// (c)1997 GOUIRAN Christophe
// 

#include "zcomponent.moc"
#include "ftp.moc"
#include <qdir.h>
#include <qfileinf.h>
#include <stdlib.h> // malloc()
#include <unistd.h> // getuid()
#include <qmessagebox.h>
#include "ftp.h"

#define MAX(a,b) ( (a>b) ? (a) : (b) )
#define MIN(a,b) ( (a<b) ? (a) : (b) )

Module::Module()
  : inComponent() 
{
    component = new ftpPlugin;
}


ZList &Module::scan(QString path)
{
    return component -> scan(path);
}


QString Module::cdUp()
{
    return component -> cdUp();
}


bool Module::exists(const QString path)
{
	return component -> exists(path);
}


QStrList Module::getMenu()
{
	return component -> getMenu();
}


void Module::menuClicked(int id)
{
	component -> menuClicked(id);
}


extern "C" inComponent *getNewModule()
{
    return new Module;
}


ftpPlugin::ftpPlugin()
{
	dotFiles = FALSE;
	firstDirs = TRUE;
	caseSensitive = TRUE;

	connected = FALSE;

	FtpInit();

  	dir.setSorting(QDir::DirsFirst | QDir::IgnoreCase);
}


QStrList ftpPlugin::getMenu()
{
	QStrList l;

	if (dotFiles) // fichiers caches
		l.append("Voir les fichiers caches_1");
	else
		l.append("Voir les fichiers caches_0");

	if (caseSensitive)
		l.append("Ignorer les minuscules/majuscules_1");
	else
		l.append("Ignorer les minuscules/majuscules_0");
	
	if (firstDirs)
		l.append("Afficher les repertoires en premier_1");
	else
		l.append("Afficher les repertoires en premier_0");
	
	return l;
}


void ftpPlugin::menuClicked(int id)
{
	if (id == 0) // voir les fichiers caches
	{
		if (dotFiles)
		{
			dotFiles = FALSE;
			dir.setFilter(QDir::Files | QDir::Dirs);
		}
		else
		{
			dotFiles = TRUE;
			dir.setFilter(QDir::Hidden | QDir::Files | QDir::Dirs);
		}
	}

	if (id == 1) // ignore maj/min
	{
		if (caseSensitive)
		{
			caseSensitive = FALSE;
			if (firstDirs)
				dir.setSorting(QDir::DirsFirst);
			else
				dir.setSorting(0);
			
		}
		else
		{
			caseSensitive = TRUE;
			if (firstDirs)
				dir.setSorting(QDir::DirsFirst | QDir::IgnoreCase);
			else
				dir.setSorting(QDir::IgnoreCase);
		}
	}

	if (id == 2) // rep en premier
	{
		if (firstDirs)
		{
			firstDirs = FALSE;
			if (caseSensitive)
				dir.setSorting(QDir::IgnoreCase);
			else
				dir.setSorting(0);
		}
		else
		{
			firstDirs = TRUE;
			if (caseSensitive)
				dir.setSorting(QDir::IgnoreCase | QDir::DirsFirst);
			else
				dir.setSorting(QDir::DirsFirst);
		}			

	}
}


bool ftpPlugin::exists(const QString path)
{
	return QFile::exists(path.data());
}


ZList &ftpPlugin::scan(const QString path)
{

	if (!connected) // on est pas encore connecte
	{
		debug("Connection au site ...");
		user = QString("anonymous");
		host = QString("localhost");
		pass = QString("cgouiran@mygale.org");

		if(!FtpConnect(host.data(), &conn))
		{
			debug("Erreur de connection");
			exit(12);
		}

		if(!FtpLogin(user.data(), pass.data(), conn))
		{
			debug("Erreur de login");
			exit(12);
		}	
	}

    ZFileInfo *fi;

	QFileInfo test(QFile(path.data()));
	if (test.isFile()) return list;

	//debug("path : '%s'", path.data());
	//debug("path : '%s'", test.absFilePath().data());
	//debug("rep : %d", test.isDir());

	//debug("ownerId() : %d", test.ownerId());
	//debug("getuid() : %d", getuid());
	//debug("permission d'execution : %d", test.permission(QFileInfo::ExeOther));

	if ((test.ownerId() != getuid()) && (!test.permission(QFileInfo::ExeOther)) && getuid()) 
	{
		QMessageBox::critical(0, "Attention", "Vous n'avez pas les droits d'acces sur ce repertoire");
		list.clear();
		return list;
	}


  	if (/*dir.path() != path && */dir.cd(path))
  	{
			
    	list.clear();

    	const QFileInfoList *List = dir.entryInfoList();
    	QFileInfoListIterator it(*List);
			
    	while( fi = (ZFileInfo *) it.current() )
    	{
      		if ((fi -> fileName() != ".") || (fi -> fileName() == ".." && (!dir.isRoot()))) list.append(fi); 

      		++it;	
    	}

  	}   

  	return list;
  
} // scan()	


QString ftpPlugin::cdUp()
{
	QDir d = dir;
	d.cdUp();	

	QFileInfo test(QFile(d.absPath()));

	debug("path : '%s'", test.absFilePath().data());
	debug("rep : %d", test.isDir());
	debug("rep : %s", d.absPath().data());
	
	return d.absPath();
}
