// Classe ZListeur du projet zfm
// (c)1997 GOUIRAN Christophe
// 

#include <qdir.h>
#include <qfileinf.h>
#include <stdlib.h> // malloc()
#if !defined(WIN32)
#include <unistd.h> // getuid()
#endif
#include <qmessagebox.h>
#include "inplugin.h"

#define MAX(a,b) ( (a>b) ? (a) : (b) )
#define MIN(a,b) ( (a<b) ? (a) : (b) )

inModule::inModule()
  : inComponent() 
{
    component = new inPlugin;
}


bool inModule::open(QString name, int mode)
{
	return component -> open(name, mode);
}


bool inModule::atEnd()
{
	return component -> atEnd();
}


int inModule::getch()
{
	return component -> getch();
}


void inModule::close()
{
	component -> close();
}


bool inModule::at(int offset)
{
	return component -> at(offset);
}


int inModule::readBlock(char *p, uint len)
{
	return component -> readBlock(p, len);
}


ZList &inModule::scan(QString path)
{
    return component -> scan(path);
}


ZListDevice &inModule::getDevices()
{
    return component -> getDevices();
}


QString inModule::cdUp()
{
    return component -> cdUp();
}


bool inModule::exists(const QString path)
{
	return component -> exists(path);
}


bool inModule::isGoodDir(QString path)
{
	return component -> isGoodDir(path);
}

bool inModule::isGoodFile(QString file)
{
	return component -> isGoodFile(file);
}

QStrList inModule::getMenu()
{
	return component -> getMenu();
}


void inModule::menuClicked(int id)
{
	component -> menuClicked(id);
}


extern "C" inComponent 
#ifdef WIN32
__declspec(dllexport)
#endif
*getNewInModule()
{
    return new inModule;
}

inPlugin::inPlugin()
{
	dotFiles = FALSE;
	firstDirs = TRUE;
	caseSensitive = TRUE;

	optionsChanged = FALSE;

  	dir.setSorting(QDir::DirsFirst | QDir::IgnoreCase);
}


QStrList inPlugin::getMenu()
{
	QStrList l;

	if (dotFiles) // fichiers caches
		l.append("View hidden files_1");
	else
		l.append("View hidden files_0");

	if (caseSensitive)
		l.append("Case unsensitive_1");
	else
		l.append("Case unsensitive_0");
	
	if (firstDirs)
		l.append("Display directories first_1");
	else
		l.append("Display directories first_0");
	
	return l;
}


void inPlugin::menuClicked(int id)
{
	if (id == 0) // voir les fichiers caches
	{
		if (dotFiles)
		{
			dotFiles = FALSE;
			dir.setFilter(QDir::Files | QDir::Dirs);
		}
		else
		{
			dotFiles = TRUE;
			dir.setFilter(QDir::Hidden | QDir::Files | QDir::Dirs);
		}
	}

	if (id == 1) // ignore maj/min
	{
		if (caseSensitive)
		{
			caseSensitive = FALSE;
			if (firstDirs)
				dir.setSorting(QDir::DirsFirst);
			else
				dir.setSorting(0);
			
		}
		else
		{
			caseSensitive = TRUE;
			if (firstDirs)
				dir.setSorting(QDir::DirsFirst | QDir::IgnoreCase);
			else
				dir.setSorting(QDir::IgnoreCase);
		}
	}

	if (id == 2) // rep en premier
	{
		if (firstDirs)
		{
			firstDirs = FALSE;
			if (caseSensitive)
				dir.setSorting(QDir::IgnoreCase);
			else
				dir.setSorting(0);
		}
		else
		{
			firstDirs = TRUE;
			if (caseSensitive)
				dir.setSorting(QDir::IgnoreCase | QDir::DirsFirst);
			else
				dir.setSorting(QDir::DirsFirst);
		}			

	}

	optionsChanged = TRUE;
}


bool inPlugin::exists(const QString path)
{
	return QFile::exists(path.data());
}


ZList &inPlugin::scan(const QString path)
{
    QFileInfo *fi;
	ZFileInfo *f;

	QFileInfo test(QFile(path.data()));
	if (test.isFile()) return list;

#if !defined(__CYGWIN32__) && !defined(WIN32)
	if ((test.ownerId() != getuid()) && (!test.permission(QFileInfo::ExeOther)) && getuid()) 
	{
		QMessageBox::critical(0, "Warning", "You don't have access to this directory");
		list.clear();
		return list;
	}
#endif

  	if (/*dir.path() != path && */dir.cd(path))
  	{
			
    	list.clear();

    	const QFileInfoList *List = dir.entryInfoList();
    	if (List == 0) return list; // for empty devices

		QFileInfoListIterator it(*List);
			
    	while( fi = it.current() )
    	{
			f = new ZFileInfo(fi);
			f -> setSelected(FALSE);
			//debug("%s", f -> absFilePath().data());
      		if (f -> fileName() != "." && f -> fileName() != "..") list.append(f);
			if (f -> fileName() == ".." && !dir.isRoot()) list.append(f); 

      		++it;	
    	}

  	}   

  	return list;
  
} // scan()	


ZListDevice &inPlugin::getDevices()
{
	ZFileInfoDevice fid;

	fid.setRootPath("/");
	fid.setFileSystem("ext2");

	devices.clear();
	devices.append(&fid);

	return devices;
}


bool inPlugin::isGoodDir(QString path)
{
	QFile t(path);
	QDir tes(path);
	//QFile tes(path);
	QFileInfo test(t);
	return !test.isFile() && tes.isReadable()
//#if !defined(__CYGWIN32__) && !defined(WIN32)		
//	&& ((test.ownerId() == getuid() && test.permission(QFileInfo::ExeUser)) || getuid() == 0);
//#else
	;
//#endif
}

bool inPlugin::isGoodFile(QString file)
{
	QFile tes(file);
	QFileInfo test(tes);
	return test.isFile()
#if !defined(__CYGWIN32__) && !defined(WIN32)
	&& ((test.ownerId() == getuid() && test.permission(QFileInfo::ReadUser)) || getuid() == 0);
#else
	;
#endif
}


QString inPlugin::cdUp()
{
	QDir d = dir;
	d.cdUp();	

	QFileInfo test(QFile(d.absPath()));

	debug("path : '%s'", test.absFilePath().data());
	debug("rep : %d", test.isDir());
	debug("rep : %s", d.absPath().data());
	
	return d.absPath();
}


bool inPlugin::open(QString name, int mode)
{
	internal.setName(name);
	return internal.open(mode);
}


bool inPlugin::atEnd()
{
	return internal.isOpen() && internal.atEnd();
}


int inPlugin::getch()
{
	if (internal.isOpen())
		return internal.getch();
	else
		return -1;
}


void inPlugin::close()
{
	if (internal.isOpen())
		internal.close();
}


bool inPlugin::at(int offset)
{
	return internal.isOpen() && internal.at(offset);
}


int inPlugin::readBlock(char *p, uint len)
{
	if (internal.isOpen())
		return internal.readBlock(p, len);
	else
		return -1;
}