/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <qpainter.h>
#include <qpixmapcache.h>
#include "toolbar.h"

ZToolbar::ZToolbar(QWidget *parent, const char *name)
    : QTableView(parent, name)
{
    NumCols = 0;
    setTableFlags( Tbl_clipCellPainting | Tbl_smoothScrolling );
    setNumRows(1);
    curCol = -1;

    connect(&timer, SIGNAL(timeout()), SLOT(timeOut()));
    toBlink = TRUE;
    currentBlink = -1;
}

void ZToolbar::clear()
{
	NumCols = 0;
	curCol = -1;

	leftList.clear();
	rightList.clear();
	leftActions.clear();
	rightActions.clear();
}

void ZToolbar::readConfig()
{
    for(int i = 0 ;; i++)
    {
	actioninfo *fi = config -> get(i);
	if (fi == 0) break;
	addEntry(fi -> leftIndex, fi -> leftbutton, fi -> rightIndex, fi -> rightbutton);
    }
}

void ZToolbar::addEntry(int leftIndex, QString leftAction, int rightIndex, QString rightAction)
{
  
    int *left = new int;	
    *left = leftIndex;
    leftList.append(left);

    int *right = new int;
    *right = rightIndex;
    rightList.append(right);	
  
    QString *lefta = new QString;
    *lefta = leftAction.copy();
    leftActions.append(lefta -> data());

    QString *righta = new QString;
    *righta = rightAction.copy();
    rightActions.append(righta -> data());

    NumCols++;    
    setNumCols(NumCols);

    updateCell(0, NumCols - 1);
}

void ZToolbar::setAction(int index, QString action)
{
	leftActions.remove(index);
	leftActions.insert(index, action.data());
}

char *ZToolbar::getAction(int index)
{
	return leftActions.at(index);
}

void ZToolbar::removeEntry(unsigned int index)
{
    if (index < 0 || index >= leftList.count()) return;

    if ((leftList.remove(index) || rightList.remove(index)) && (leftActions.remove(index) || rightActions.remove(index))) 
    {
		setNumCols(--NumCols);
		repaint();
    }
}

void ZToolbar::paintCell(QPainter *p, int row, int col)
{
    QPixmap pi;

    int *index = leftList.at(col);

	if (*index == -1)
		return; // might never happen !!

    pi.loadFromData(pixmapInfo[*index].pixmap, pixmapInfo[*index].size);

    p -> drawPixmap(0, 0, pi);

    if (col == curCol) // the button is being pressed
    {
	p -> setPen(QColor(0, 0, 0));
	p -> drawLine(0, 0, cellWidth(col), 0);
	p -> drawLine(0, 0, 0, cellHeight(col));
	
	p -> setPen(QColor(255, 255, 255));
	p -> drawLine(cellWidth(col) - 1, 0, cellWidth(col) - 1, cellHeight(col));
	p -> drawLine(0, cellHeight(col) - 1, cellWidth(col), cellHeight(col) - 1);
    }

    if (currentBlink == col)
    {
        if (toBlink == TRUE)
        {
            p -> eraseRect(2, 2, 24 - 4, 16 - 4);
            toBlink = FALSE;
        }
        else
            toBlink = TRUE;
    }
}

void ZToolbar::resizeEvent(QResizeEvent *e)
{
   setNumCols(NumCols);
    setNumRows(1);

    QTableView::resizeEvent(e);
}

void ZToolbar::mousePressEvent(QMouseEvent *e)
{
    if (e -> button() == LeftButton)
    {
	QPoint clickedPos = e -> pos();
	curCol = findCol(clickedPos.x());

	updateCell(0, curCol);
    }
}

void ZToolbar::mouseReleaseEvent(QMouseEvent *e)
{
    if (e -> button() == LeftButton)
    {
	QPoint clickedPos = e -> pos();
	int col = findCol(clickedPos.x());

	if (curCol == col)
	{
	    curCol = -1;
	    updateCell(0, col);
	}
	else if (curCol != col)
	{
	    int newCol = curCol;
	    curCol = -1;
	    updateCell(0, newCol);
	}

	emit(clicked(col));
    }
}	

int ZToolbar::getNumEntry(int index)
{
    return *leftList.at(index);
}

void ZToolbar::timeOut()
{
    updateCell(0, currentBlink);
}

void ZToolbar::blink(int index)
{
    int oldBlink = currentBlink;
    currentBlink = index;

    if (oldBlink != -1)
        updateCell(0, oldBlink);

    if (index == -1)
        timer.stop();
    else 
    {
        toBlink = TRUE;
        timer.start(500);
    }

}
