/* This file is part of the zfm
    Copyright (C) 1998-99 GOUIRAN Christophe (Christophe.Gouiran@lpl.univ-aix.fr)
                          BELLET Vincent (vbellet@cybercable.fr)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
/*  utils.cpp
    projet : zdesktop
    auteur : BELLET Vincent, GOUIRAN Christophe
    description : primitive utilisee pour le chargement de composants */

#ifndef WIN32
extern "C" {
#include <dlfcn.h>
}

#include <unistd.h>
#else
#include <windows.h>
#include <winbase.h>
#endif

#include <qfile.h>
#include <iostream.h>
#include <qstrlist.h>
#include "utils.h"


void *ZModule::loadModule(QString name)
{
	//QFile::exit libtoolfile(name);

    if (!QFile::exists(name))
    {
		debug("No such file %s", name.data());
		return 0;
    }
  
    debug("Loading plugins '%s'", name.data());

    void *handle = 
#ifdef WIN32
		LoadLibrary(name.data());
#else		
		dlopen(name.data(), RTLD_LAZY);
#endif
    if (handle)
		return handle;
    else
    {
		QString error;
#ifdef WIN32
		LPVOID lpMsgBuf;FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,    NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,    0,    NULL);
		debug("%s", lpMsgBuf);
#else
		error = dlerror();
		debug("%s", error.data());
#endif
		return 0;
    }
}


outComponent *ZModule::getOutModule(void *handle)
{
    // pointeur sur une fonction
    outComponent *(*get)();

    if (handle)
    {
		// get module instance
		get = (outComponent*(*)())
#ifdef WIN32
		GetProcAddress(handle, "getNewOutModule");
#else		
		dlsym(handle, "getNewOutModule");
#endif
		if (get)
	    	return get();
		else
		{
			QString error;
#ifdef WIN32
		LPVOID lpMsgBuf;FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,    NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,    0,    NULL);
		debug("%s", lpMsgBuf);
#else
		error = dlerror();
		debug("%s", error.data());
#endif
			//QString error = dlerror();
			//debug("%s", error.data());
		}
    }

    return 0;
}

#if !defined(__CYGWIN32__)
outGLComponent *ZModule::getOutGLModule(void *handle)
{
    // pointeur sur une fonction
    outGLComponent *(*get)();

    if (handle)
    {
		// get module instance
		get = (outGLComponent*(*)())
#ifdef WIN32
		GetProcAddress (handle, "getNewOutGLModule");
#else		
		dlsym(handle, "getNewOutGLModule");
#endif
		if (get)
	    	return get();
		else
		{
			QString error;
#ifdef WIN32
		LPVOID lpMsgBuf;FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,    NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,    0,    NULL);
		debug("%s", lpMsgBuf);
#else
		error = dlerror();
		debug("%s", error.data());
#endif
			
		}
    }

    return 0;
}
#endif

inComponent *ZModule::getInModule(void *handle)
{
    // pointeur sur une fonction
    inComponent *(*get)();

    if (handle)
    {
		// get module instance
		get = (inComponent*(*)())
#ifdef WIN32
		GetProcAddress (handle, "getNewInModule");
#else		
			dlsym(handle, "getNewInModule");
#endif
			//get = dlsym(handle, "getNewModule");

		if (get)
	    	return get();
		else
		{
			QString error;
#ifdef WIN32
		LPVOID lpMsgBuf;FormatMessage( 
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,    NULL,
		GetLastError(),
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf,    0,    NULL);
		debug("%s", lpMsgBuf);
#else
		error = dlerror();
		debug("%s", error.data());
#endif
			//QString error = dlerror();
			//debug("%s", error.data());
		}
    }

    return 0;
}


void ZModule::unloadModule(void *handle)
{
    // close lib
    //cout << "fermeture du module" << endl;
    if (handle)
#ifdef WIN32
		FreeLibrary(handle);
#else
		dlclose(handle);
#endif
}

#ifndef WIN32
bool ZModule::hasRootRights()
{
	return (geteuid() == 0);
}
#endif